@@ -9,11 +9,11 @@ var path = require('path');
 // and highlights zlib/inflate bottleneck (compare "png" format to "png8")
 var port = 8000;
 
-var im = new mapnik.Image(256,256)
+var im = new mapnik.Image(256, 256);
 
 http.createServer(function(req, res) {
     res.writeHead(200, {'Content-Type': 'image/png'});
-    res.end(im.encodeSync("png8"));
+    res.end(im.encodeSync('png8'));
 }).listen(port);
 
-console.log("server running on port " + port);
+console.log('server running on port ' + port);
