@@ -1,6 +1,6 @@
 #!/usr/bin/env node
 
-// This example shows how to use node-mapnik to render 
+// This example shows how to use node-mapnik to render
 // a map to a file
 //
 // expected output: http://goo.gl/cyGwo
@@ -11,13 +11,13 @@ var child_process = require('child_process');
 
 var usage = 'usage: render.js <stylesheet> <image>';
 
-var stylesheet = process.ARGV[2];
+var stylesheet = process.argv[2];
 if (!stylesheet) {
    console.log(usage);
    process.exit(1);
 }
 
-var image = process.ARGV[3];
+var image = process.argv[3];
 if (!image) {
    console.log(usage);
    process.exit(1);
