@@ -26,7 +26,9 @@ class Feature: public node::ObjectWrap {
     static Handle<Value> extent(const Arguments &args);
     static Handle<Value> attributes(const Arguments &args);
     static Handle<Value> addGeometry(const Arguments &args);
+    static Handle<Value> addAttributes(const Arguments &args);
     static Handle<Value> numGeometries(const Arguments &args);
+    static Handle<Value> toString(const Arguments &args);
 
     // todo
     // how to allow altering of attributes
@@ -38,6 +40,7 @@ class Feature: public node::ObjectWrap {
   private:
     ~Feature();
     mapnik::feature_ptr this_;
+    mapnik::context_ptr ctx_;
 };
 
 #endif
