@@ -20,7 +20,7 @@
 #include <mapnik/map.hpp>
 #include <mapnik/projection.hpp>
 #include <mapnik/layer.hpp>
-#include <mapnik/filter_factory.hpp>
+#include <mapnik/expression.hpp>
 #include <mapnik/image_util.hpp>
 #include <mapnik/config_error.hpp>
 #include <mapnik/load_map.hpp>
@@ -49,6 +49,7 @@
 #include "mapnik_palette.hpp"
 #include "mapnik_color.hpp"
 
+
 Persistent<FunctionTemplate> Map::constructor;
 
 void Map::Initialize(Handle<Object> target) {
@@ -99,6 +100,9 @@ void Map::Initialize(Handle<Object> target) {
     ATTR(constructor, "extent", get_prop, set_prop);
     ATTR(constructor, "maximumExtent", get_prop, set_prop);
     ATTR(constructor, "background", get_prop, set_prop);
+    ATTR(constructor, "parameters", get_prop, set_prop);
+
+    NODE_SET_PROTOTYPE_METHOD(constructor, "size", size);
 
     target->Set(String::NewSymbol("Map"),constructor->GetFunction());
     //eio_set_max_poll_reqs(10);
@@ -108,26 +112,26 @@ void Map::Initialize(Handle<Object> target) {
 Map::Map(int width, int height) :
   ObjectWrap(),
   map_(boost::make_shared<mapnik::Map>(width,height)),
-  in_use_(0) {}
+  in_use_(0),
+  estimated_size_(0) {}
 
 Map::Map(int width, int height, std::string const& srs) :
   ObjectWrap(),
   map_(boost::make_shared<mapnik::Map>(width,height,srs)),
-  in_use_(0) {}
+  in_use_(0),
+  estimated_size_(0) {}
 
 Map::~Map()
 {
-    // std::clog << "~Map(node)\n";
-    // release is handled by boost::shared_ptr
+    if (estimated_size_ >0)
+        V8::AdjustAmountOfExternalAllocatedMemory(-estimated_size_);
 }
 
 void Map::acquire() {
-    //std::cerr << "acquiring!!\n";
     ++in_use_;
 }
 
 void Map::release() {
-    //std::cerr << "releasing!!\n";
     --in_use_;
 }
 
@@ -164,7 +168,7 @@ Handle<Value> Map::New(const Arguments& args)
                String::New("'width' and 'height' must be a integers")));
         if (!args[2]->IsString())
             return ThrowException(Exception::Error(
-               String::New("'srs' value must be a string")));            
+               String::New("'srs' value must be a string")));
         Map* m = new Map(args[0]->IntegerValue(),args[1]->IntegerValue(),TOSTR(args[2]));
         m->Wrap(args.This());
         return args.This();
@@ -174,10 +178,98 @@ Handle<Value> Map::New(const Arguments& args)
         return ThrowException(Exception::Error(
           String::New("please provide Map width and height and optional srs")));
     }
-    //return args.This();
     return Undefined();
 }
 
+class sizeof_symbolizer : public boost::static_visitor<>
+{
+public:
+    sizeof_symbolizer( unsigned int * usage):
+        usage_(usage),
+        factor_(21 /*arbitrary*/) {}
+
+    void operator () ( mapnik::point_symbolizer const& sym )
+    {
+        *usage_ += (sizeof(sym)*factor_);
+    }
+
+    void operator () ( mapnik::line_symbolizer const& sym )
+    {
+        *usage_ += (sizeof(sym)*factor_);
+    }
+
+    void operator () ( mapnik::line_pattern_symbolizer const& sym )
+    {
+        *usage_ += (sizeof(sym)*factor_);
+    }
+
+    void operator () ( mapnik::polygon_symbolizer const& sym )
+    {
+        *usage_ += (sizeof(sym)*factor_);
+    }
+
+    void operator () ( mapnik::polygon_pattern_symbolizer const& sym )
+    {
+        *usage_ += (sizeof(sym)*factor_);
+    }
+
+    void operator () ( mapnik::raster_symbolizer const& sym )
+    {
+        *usage_ += (sizeof(sym)*factor_);
+    }
+
+    void operator () ( mapnik::shield_symbolizer const& sym )
+    {
+        *usage_ += (sizeof(sym)*factor_);
+    }
+
+    void operator () ( mapnik::text_symbolizer const& sym )
+    {
+        *usage_ += (sizeof(sym)*factor_);
+    }
+
+    void operator () ( mapnik::building_symbolizer const& sym )
+    {
+        *usage_ += (sizeof(sym)*factor_);
+    }
+
+    void operator () ( mapnik::markers_symbolizer const& sym)
+    {
+        *usage_ += (sizeof(sym)*factor_);
+    }
+
+    unsigned int * usage_;
+    unsigned int factor_;
+};
+
+unsigned int Map::estimate_map_size()
+{
+    // very rough estimate of memory usage of a map
+    unsigned int mem_usage(0);
+    mapnik::Map::const_style_iterator sty_itr = map_->styles().begin();
+    for (; sty_itr != map_->styles().end(); ++sty_itr)
+    {
+        mapnik::feature_type_style const& style = sty_itr->second;
+        mapnik::rules::const_iterator rule_itr = style.get_rules().begin();
+        for (; rule_itr != style.get_rules().end(); ++rule_itr)
+        {
+            mapnik::rule::symbolizers::const_iterator begin = rule_itr->get_symbolizers().begin();
+            mapnik::rule::symbolizers::const_iterator end = rule_itr->get_symbolizers().end();
+            sizeof_symbolizer detector( &mem_usage);
+            std::for_each( begin, end , boost::apply_visitor( detector ));
+        }
+    }
+    estimated_size_ = mem_usage;
+    return mem_usage;
+}
+
+Handle<Value> Map::size(const Arguments& args)
+{
+    HandleScope scope;
+    Map* m = ObjectWrap::Unwrap<Map>(args.This());
+    return scope.Close(Integer::New(m->estimate_map_size()));
+}
+
 Handle<Value> Map::get_prop(Local<String> property,
                          const AccessorInfo& info)
 {
@@ -219,6 +311,18 @@ Handle<Value> Map::get_prop(Local<String> property,
         else
             return Undefined();
     }
+    else if (a == "parameters") {
+        Local<Object> ds = Object::New();
+        mapnik::parameters const& params = m->map_->get_extra_parameters();
+        mapnik::parameters::const_iterator it = params.begin();
+        mapnik::parameters::const_iterator end = params.end();
+        for (; it != end; ++it)
+        {
+            node_mapnik::params_to_object serializer( ds , it->first);
+            boost::apply_visitor( serializer, it->second );
+        }
+        return scope.Close(ds);
+    }
     return Undefined();
 }
 
@@ -288,14 +392,48 @@ void Map::set_prop(Local<String> property,
         if (!value->IsObject())
             ThrowException(Exception::TypeError(
               String::New("mapnik.Color expected")));
-    
+
         Local<Object> obj = value->ToObject();
         if (obj->IsNull() || obj->IsUndefined() || !Color::constructor->HasInstance(obj))
             ThrowException(Exception::TypeError(String::New("mapnik.Color expected")));
         Color *c = ObjectWrap::Unwrap<Color>(obj);
         m->map_->set_background(*c->get());
     }
+    else if (a == "parameters") {
+        if (!value->IsObject())
+            ThrowException(Exception::TypeError(
+              String::New("object expected for map.parameters")));
 
+        Local<Object> obj = value->ToObject();
+        if (obj->IsNull() || obj->IsUndefined())
+            ThrowException(Exception::TypeError(String::New("object expected for map.parameters, cannot be null/undefined")));
+
+        mapnik::parameters params;
+        Local<Array> names = obj->GetPropertyNames();
+        uint32_t i = 0;
+        uint32_t a_length = names->Length();
+        while (i < a_length) {
+            Local<Value> name = names->Get(i)->ToString();
+            Local<Value> value = obj->Get(name);
+            if (value->IsString()) {
+                params[TOSTR(name)] = TOSTR(value);            
+            } else if (value->IsNumber()) {
+                double num = value->NumberValue();
+                // todo - round
+                if (num == value->IntegerValue()) {
+                    int integer = value->IntegerValue();
+                    params[TOSTR(name)] = integer;
+                } else {
+                    double dub_val = value->NumberValue();
+                    params[TOSTR(name)] = dub_val;
+                }
+            } else {
+                std::clog << "unhandled type for property: " << TOSTR(name) << "\n";
+            }
+            i++;
+        }
+        m->map_->set_extra_parameters(params);
+    }
 }
 
 Handle<Value> Map::scaleDenominator(const Arguments& args)
@@ -346,7 +484,7 @@ Handle<Value> Map::get_layer(const Arguments& args)
         {
           return ThrowException(Exception::TypeError(
             String::New("invalid layer index")));
-        }    
+        }
     }
     else if (layer->IsString())
     {
@@ -360,16 +498,16 @@ Handle<Value> Map::get_layer(const Arguments& args)
                 found = true;
                 return scope.Close(Layer::New(layers[idx]));
             }
-            ++idx; 
+            ++idx;
         }
-        if (!found) 
+        if (!found)
         {
             std::ostringstream s;
             s << "Layer name '" << layer_name << "' not found";
             return ThrowException(Exception::TypeError(
               String::New(s.str().c_str())));
         }
-    
+
     }
     else
     {
@@ -524,6 +662,7 @@ Handle<Value> Map::resize(const Arguments& args)
 
 
 typedef struct {
+    uv_work_t request;
     Map *m;
     std::string stylesheet;
     bool strict;
@@ -574,20 +713,19 @@ Handle<Value> Map::load(const Arguments& args)
     Map* m = ObjectWrap::Unwrap<Map>(args.This());
 
     load_image_baton_t *closure = new load_image_baton_t();
-    
+    closure->request.data = closure;
     closure->stylesheet = TOSTR(stylesheet);
     closure->m = m;
     closure->strict = strict;
     closure->error = false;
     closure->cb = Persistent<Function>::New(Handle<Function>::Cast(callback));
-    eio_custom(EIO_Load, EIO_PRI_DEFAULT, EIO_AfterLoad, closure);
-    ev_ref(EV_DEFAULT_UC);
+    uv_queue_work(uv_default_loop(), &closure->request, EIO_Load, EIO_AfterLoad);
     m->Ref();
-    
+    uv_ref(uv_default_loop());
     return Undefined();
 }
 
-int Map::EIO_Load(eio_req *req)
+void Map::EIO_Load(uv_work_t* req)
 {
     load_image_baton_t *closure = static_cast<load_image_baton_t *>(req->data);
 
@@ -605,15 +743,13 @@ int Map::EIO_Load(eio_req *req)
         closure->error = true;
         closure->error_name = "unknown exception happened while rendering the map,\n this should not happen, please submit a bug report";
     }
-    return 0;
 }
 
-int Map::EIO_AfterLoad(eio_req *req)
+void Map::EIO_AfterLoad(uv_work_t* req)
 {
     HandleScope scope;
 
     load_image_baton_t *closure = static_cast<load_image_baton_t *>(req->data);
-    ev_unref(EV_DEFAULT_UC);
 
     TryCatch try_catch;
 
@@ -623,6 +759,7 @@ int Map::EIO_AfterLoad(eio_req *req)
     } else {
         Local<Value> argv[2] = { Local<Value>::New(Null()), Local<Value>::New(closure->m->handle_) };
         closure->cb->Call(Context::GetCurrent()->Global(), 2, argv);
+        V8::AdjustAmountOfExternalAllocatedMemory(closure->m->estimate_map_size());
     }
 
     if (try_catch.HasCaught()) {
@@ -630,19 +767,23 @@ int Map::EIO_AfterLoad(eio_req *req)
     }
 
     closure->m->Unref();
+    uv_unref(uv_default_loop());
     closure->cb.Dispose();
     delete closure;
-    return 0;
 }
 
 
 Handle<Value> Map::loadSync(const Arguments& args)
 {
     HandleScope scope;
-    if (args.Length() != 1 || !args[0]->IsString())
+    if (!args[0]->IsString())
       return ThrowException(Exception::TypeError(
         String::New("first argument must be a path to a mapnik stylesheet")));
 
+    if (args.Length() != 1)
+      return ThrowException(Exception::TypeError(
+        String::New("only accepts one argument: a path to a mapnik stylesheet")));
+
     Map* m = ObjectWrap::Unwrap<Map>(args.This());
     std::string const& stylesheet = TOSTR(args[0]);
     bool strict = false;
@@ -660,6 +801,7 @@ Handle<Value> Map::loadSync(const Arguments& args)
       return ThrowException(Exception::TypeError(
         String::New("something went wrong loading the map")));
     }
+    V8::AdjustAmountOfExternalAllocatedMemory(m->estimate_map_size());
     return Undefined();
 }
 
@@ -685,9 +827,9 @@ Handle<Value> Map::fromStringSync(const Arguments& args)
         if (!args[1]->IsObject())
             return ThrowException(Exception::TypeError(
               String::New("options must be an object, eg {strict: true, base: \".\"'}")));
-    
+
         Local<Object> options = args[1]->ToObject();
-    
+
         Local<String> param = String::New("strict");
         if (options->Has(param))
         {
@@ -697,7 +839,7 @@ Handle<Value> Map::fromStringSync(const Arguments& args)
                 String::New("'strict' must be a Boolean")));
             strict = param_val->BooleanValue();
         }
-    
+
         param = String::New("base");
         if (options->Has(param))
         {
@@ -708,7 +850,7 @@ Handle<Value> Map::fromStringSync(const Arguments& args)
             base_path = TOSTR(param_val);
         }
     }
-    
+
     Map* m = ObjectWrap::Unwrap<Map>(args.This());
 
     std::string const& stylesheet = TOSTR(args[0]);
@@ -727,10 +869,12 @@ Handle<Value> Map::fromStringSync(const Arguments& args)
       return ThrowException(Exception::TypeError(
         String::New("something went wrong loading the map")));
     }
+    V8::AdjustAmountOfExternalAllocatedMemory(m->estimate_map_size());
     return Undefined();
 }
 
 typedef struct {
+    uv_work_t request;
     Map *m;
     std::string stylesheet;
     std::string base_url;
@@ -782,6 +926,7 @@ Handle<Value> Map::fromString(const Arguments& args)
     Map* m = ObjectWrap::Unwrap<Map>(args.This());
 
     load_string_image_baton_t *closure = new load_string_image_baton_t();
+    closure->request.data = closure;
 
     param = String::New("base");
     if (options->Has(param))
@@ -792,20 +937,19 @@ Handle<Value> Map::fromString(const Arguments& args)
             String::New("'base' must be a string representing a filesystem path")));
         closure->base_url = TOSTR(param_val);
     }
-    
+
     closure->stylesheet = TOSTR(stylesheet);
     closure->m = m;
     closure->strict = strict;
     closure->error = false;
     closure->cb = Persistent<Function>::New(Handle<Function>::Cast(callback));
-    eio_custom(EIO_FromString, EIO_PRI_DEFAULT, EIO_AfterFromString, closure);
-    ev_ref(EV_DEFAULT_UC);
+    uv_queue_work(uv_default_loop(), &closure->request, EIO_FromString, EIO_AfterFromString);
     m->Ref();
-    
+    uv_ref(uv_default_loop());
     return Undefined();
 }
 
-int Map::EIO_FromString(eio_req *req)
+void Map::EIO_FromString(uv_work_t* req)
 {
     load_string_image_baton_t *closure = static_cast<load_string_image_baton_t *>(req->data);
 
@@ -823,15 +967,13 @@ int Map::EIO_FromString(eio_req *req)
         closure->error = true;
         closure->error_name = "unknown exception happened while rendering the map,\n this should not happen, please submit a bug report";
     }
-    return 0;
 }
 
-int Map::EIO_AfterFromString(eio_req *req)
+void Map::EIO_AfterFromString(uv_work_t* req)
 {
     HandleScope scope;
 
     load_string_image_baton_t *closure = static_cast<load_string_image_baton_t *>(req->data);
-    ev_unref(EV_DEFAULT_UC);
 
     TryCatch try_catch;
 
@@ -841,6 +983,7 @@ int Map::EIO_AfterFromString(eio_req *req)
     } else {
         Local<Value> argv[2] = { Local<Value>::New(Null()), Local<Value>::New(closure->m->handle_) };
         closure->cb->Call(Context::GetCurrent()->Global(), 2, argv);
+        V8::AdjustAmountOfExternalAllocatedMemory(closure->m->estimate_map_size());
     }
 
     if (try_catch.HasCaught()) {
@@ -848,9 +991,9 @@ int Map::EIO_AfterFromString(eio_req *req)
     }
 
     closure->m->Unref();
+    uv_unref(uv_default_loop());
     closure->cb.Dispose();
     delete closure;
-    return 0;
 }
 
 
@@ -934,6 +1077,7 @@ Handle<Value> Map::zoomToBox(const Arguments& args)
 }
 
 typedef struct {
+    uv_work_t request;
     Map *m;
     Image *im;
     double scale_factor;
@@ -945,6 +1089,7 @@ typedef struct {
 } image_baton_t;
 
 typedef struct {
+    uv_work_t request;
     Map *m;
     Grid *g;
     std::size_t layer_idx;
@@ -960,13 +1105,13 @@ typedef struct {
 Handle<Value> Map::render(const Arguments& args)
 {
     HandleScope scope;
-    
+
     // ensure at least 2 args
     if (!args.Length() >= 2) {
         return ThrowException(Exception::TypeError(
           String::New("requires at least two arguments, a renderable mapnik object, and a callback")));
     }
-    
+
     // ensure renderable object
     if (!args[0]->IsObject()) {
         return ThrowException(Exception::TypeError(
@@ -986,18 +1131,18 @@ Handle<Value> Map::render(const Arguments& args)
           << m->active()
           << " other thread(s) which is not allowed."
           << " You need to use a map pool to avoid sharing map objects between concurrent rendering";
-        std::cerr << s.str() << "\n";
+        std::clog << s.str() << "\n";
     }
-    
+
     // parse options
 
     // defaults
     double scale_factor = 1.0;
     unsigned offset_x = 0;
     unsigned offset_y = 0;
-    
+
     Local<Object> options;
-    
+
     if (args.Length() > 2) {
 
         // options object
@@ -1042,6 +1187,7 @@ Handle<Value> Map::render(const Arguments& args)
     if (Image::constructor->HasInstance(obj)) {
 
         image_baton_t *closure = new image_baton_t();
+        closure->request.data = closure;
         closure->m = m;
         closure->im = ObjectWrap::Unwrap<Image>(obj);
         closure->im->_ref();
@@ -1050,20 +1196,20 @@ Handle<Value> Map::render(const Arguments& args)
         closure->offset_y = offset_y;
         closure->error = false;
         closure->cb = Persistent<Function>::New(Handle<Function>::Cast(args[args.Length()-1]));
-        eio_custom(EIO_RenderImage, EIO_PRI_DEFAULT, EIO_AfterRenderImage, closure);
+        uv_queue_work(uv_default_loop(), &closure->request, EIO_RenderImage, EIO_AfterRenderImage);
 
     } else if (Grid::constructor->HasInstance(obj)) {
 
         Grid * g = ObjectWrap::Unwrap<Grid>(obj);
 
         std::size_t layer_idx = 0;
-        
+
         // grid requires special options for now
         if (!options->Has(String::New("layer"))) {
             return ThrowException(Exception::TypeError(
               String::New("'layer' option required for grid rendering and must be either a layer name(string) or layer index (integer)")));
         } else {
-                    
+
             std::vector<mapnik::layer> const& layers = m->map_->layers();
 
             Local<Value> layer_id = options->Get(String::New("layer"));
@@ -1083,9 +1229,9 @@ Handle<Value> Map::render(const Arguments& args)
                         layer_idx = idx;
                         break;
                     }
-                    ++idx; 
+                    ++idx;
                 }
-                if (!found) 
+                if (!found)
                 {
                     std::ostringstream s;
                     s << "Layer name '" << layer_name << "' not found";
@@ -1094,13 +1240,20 @@ Handle<Value> Map::render(const Arguments& args)
             } else if (layer_id->IsNumber()) {
                 layer_idx = layer_id->IntegerValue();
                 std::size_t layer_num = layers.size();
-            
+
                 if (layer_idx >= layer_num) {
                     std::ostringstream s;
-                    s << "Zero-based layer index '" << layer_idx << "' not valid, only '"
-                      << layers.size() << "' layers are in map";
+                    s << "Zero-based layer index '" << layer_idx << "' not valid, ";
+                    if (layer_num > 0)
+                    {
+                        s << "only '" << layers.size() << "' layers exist in map";
+                    }
+                    else
+                    {
+                        s << "no layers found in map";
+                    }
                     return ThrowException(Exception::TypeError(String::New(s.str().c_str())));
-                }    
+                }
             } else {
                 return ThrowException(Exception::TypeError(String::New("layer id must be a string or index number")));
             }
@@ -1125,6 +1278,7 @@ Handle<Value> Map::render(const Arguments& args)
         }
 
         grid_baton_t *closure = new grid_baton_t();
+        closure->request.data = closure;
         closure->m = m;
         closure->g = g;
         closure->g->_ref();
@@ -1134,33 +1288,31 @@ Handle<Value> Map::render(const Arguments& args)
         closure->offset_y = offset_y;
         closure->error = false;
         closure->cb = Persistent<Function>::New(Handle<Function>::Cast(args[args.Length()-1]));
-        eio_custom(EIO_RenderGrid, EIO_PRI_DEFAULT, EIO_AfterRenderGrid, closure);
-    
-    // TODO - canvas 
+        uv_queue_work(uv_default_loop(), &closure->request, EIO_RenderGrid, EIO_AfterRenderGrid);
     } else {
         return ThrowException(Exception::TypeError(String::New("renderable mapnik object expected")));
     }
 
-    ev_ref(EV_DEFAULT_UC);
     m->acquire();
     m->Ref();
+    uv_ref(uv_default_loop());
     return Undefined();
 }
 
-int Map::EIO_RenderGrid(eio_req *req)
+void Map::EIO_RenderGrid(uv_work_t* req)
 {
 
     grid_baton_t *closure = static_cast<grid_baton_t *>(req->data);
 
     std::vector<mapnik::layer> const& layers = closure->m->map_->layers();
-    
+
     try
     {
         // copy property names
         std::set<std::string> attributes = closure->g->get()->property_names();
-        
+
         std::string join_field = closure->g->get()->get_key();
-        if (join_field == closure->g->get()->id_name_) 
+        if (join_field == closure->g->get()->key_name())
         {
             // TODO - should feature.id() be a first class attribute?
             if (attributes.find(join_field) != attributes.end())
@@ -1180,7 +1332,7 @@ int Map::EIO_RenderGrid(eio_req *req)
                 closure->offset_y);
         mapnik::layer const& layer = layers[closure->layer_idx];
         ren.apply(layer,attributes);
-    
+
     }
     catch (const std::exception & ex)
     {
@@ -1192,18 +1344,14 @@ int Map::EIO_RenderGrid(eio_req *req)
         closure->error = true;
         closure->error_name = "Unknown error occured, please file bug";
     }
-
-    return 0;
-
 }
 
 
-int Map::EIO_AfterRenderGrid(eio_req *req)
+void Map::EIO_AfterRenderGrid(uv_work_t* req)
 {
     HandleScope scope;
 
     grid_baton_t *closure = static_cast<grid_baton_t *>(req->data);
-    ev_unref(EV_DEFAULT_UC);
 
     TryCatch try_catch;
 
@@ -1223,13 +1371,13 @@ int Map::EIO_AfterRenderGrid(eio_req *req)
 
     closure->m->release();
     closure->m->Unref();
+    uv_unref(uv_default_loop());
     closure->g->_unref();
     closure->cb.Dispose();
     delete closure;
-    return 0;
 }
 
-int Map::EIO_RenderImage(eio_req *req)
+void Map::EIO_RenderImage(uv_work_t* req)
 {
     image_baton_t *closure = static_cast<image_baton_t *>(req->data);
 
@@ -1252,15 +1400,13 @@ int Map::EIO_RenderImage(eio_req *req)
         closure->error = true;
         closure->error_name = "unknown exception happened while rendering the map,\n this should not happen, please submit a bug report";
     }
-    return 0;
 }
 
-int Map::EIO_AfterRenderImage(eio_req *req)
+void Map::EIO_AfterRenderImage(uv_work_t* req)
 {
     HandleScope scope;
 
     image_baton_t *closure = static_cast<image_baton_t *>(req->data);
-    ev_unref(EV_DEFAULT_UC);
 
     TryCatch try_catch;
 
@@ -1278,34 +1424,178 @@ int Map::EIO_AfterRenderImage(eio_req *req)
 
     closure->m->release();
     closure->m->Unref();
+    uv_unref(uv_default_loop());
     closure->im->_unref();
     closure->cb.Dispose();
     delete closure;
-    return 0;
 }
 
+typedef struct {
+    uv_work_t request;
+    Map *m;
+    std::string format;
+    std::string output;
+    palette_ptr palette;
+    bool error;
+    std::string error_name;
+    Persistent<Function> cb;
+} render_file_baton_t;
 
 Handle<Value> Map::renderFile(const Arguments& args)
 {
     HandleScope scope;
-    
-    //Map* m = ObjectWrap::Unwrap<Map>(args.This());
-    // TODO
-    return ThrowException(Exception::Error(
-        String::New("sorry! renderFile async is not yet implemented")));
-    //return Undefined();
+
+    if (!args.Length() >= 1 || !args[0]->IsString())
+      return ThrowException(Exception::TypeError(
+        String::New("first argument must be a path to a file to save")));
+
+    std::string format = "png";
+    palette_ptr palette;
+
+    Local<Value> callback = args[args.Length()-1];
+
+    if (!callback->IsFunction())
+        return ThrowException(Exception::TypeError(
+                    String::New("last argument must be a callback function")));
+
+    if (!args[1]->IsFunction() && args[1]->IsObject()) {
+        Local<Object> options = args[1]->ToObject();
+        if (options->Has(String::New("format")))
+        {
+            Local<Value> format_opt = options->Get(String::New("format"));
+            if (!format_opt->IsString())
+              return ThrowException(Exception::TypeError(
+                String::New("'format' must be a String")));
+
+            format = TOSTR(format_opt);
+        }
+
+        if (options->Has(String::New("palette")))
+        {
+            Local<Value> format_opt = options->Get(String::New("palette"));
+            if (!format_opt->IsObject())
+              return ThrowException(Exception::TypeError(
+                String::New("'palette' must be an object")));
+
+            Local<Object> obj = format_opt->ToObject();
+            if (obj->IsNull() || obj->IsUndefined() || !Palette::constructor->HasInstance(obj))
+              return ThrowException(Exception::TypeError(String::New("mapnik.Palette expected as second arg")));
+
+            palette = ObjectWrap::Unwrap<Palette>(obj)->palette();
+        }
+
+    } else if (!args[1]->IsFunction()) {
+        return ThrowException(Exception::TypeError(
+                    String::New("optional argument must be an object")));
+    }
+
+    Map* m = ObjectWrap::Unwrap<Map>(args.This());
+    std::string const& output = TOSTR(args[0]);
+
+    //maybe do this in the async part?
+    if (format.empty()) {
+        format = mapnik::guess_type(output);
+        if (format == "<unknown>") {
+            std::ostringstream s("");
+            s << "unknown output extension for: " << output << "\n";
+            return ThrowException(Exception::Error(
+                String::New(s.str().c_str())));
+        }
+    }
+
+    if (format == "pdf" || format == "svg" || format == "ps" || format == "ARGB32" || format == "RGB24") {
+#if defined(HAVE_CAIRO)
+#else
+        std::ostringstream s("");
+        s << "Cairo backend is not available, cannot write to " << format << "\n";
+        return ThrowException(Exception::Error(
+          String::New(s.str().c_str())));
+#endif
+    }
+
+    render_file_baton_t *closure = new render_file_baton_t();
+    closure->request.data = closure;
+
+    closure->m = m;
+    closure->error = false;
+    closure->cb = Persistent<Function>::New(Handle<Function>::Cast(callback));
+
+    closure->format = format;
+    closure->palette = palette;
+    closure->output = output;
+
+    uv_queue_work(uv_default_loop(), &closure->request, EIO_RenderFile, EIO_AfterRenderFile);
+    uv_ref(uv_default_loop());
+    m->Ref();
+
+    return Undefined();
+
 }
 
-int Map::EIO_RenderFile(eio_req *req)
+void Map::EIO_RenderFile(uv_work_t* req)
 {
-    return 0;
+    render_file_baton_t *closure = static_cast<render_file_baton_t *>(req->data);
+
+    try
+    {
+        if(closure->format == "pdf" || closure->format == "svg" || closure->format == "ps" || closure->format == "ARGB32" || closure->format == "RGB24") {
+#if defined(HAVE_CAIRO)
+            mapnik::save_to_cairo_file(*closure->m->map_,closure->output,closure->format);
+#else
+
+#endif
+        }
+        else
+        {
+            mapnik::image_32 im(closure->m->map_->width(),closure->m->map_->height());
+            mapnik::agg_renderer<mapnik::image_32> ren(*closure->m->map_,im);
+            ren.apply();
+
+            if (closure->palette.get()) {
+                mapnik::save_to_file<mapnik::image_data_32>(im.data(),closure->output,*closure->palette);
+            } else {
+                mapnik::save_to_file<mapnik::image_data_32>(im.data(),closure->output);
+            }
+        }
+    }
+    catch (const std::exception & ex)
+    {
+        closure->error = true;
+        closure->error_name = ex.what();
+    }
+    catch (...)
+    {
+        closure->error = true;
+        closure->error_name = "unknown exception happend while rendering image to file,\n this should not happen, please submit a bug report";
+    }
 }
 
-int Map::EIO_AfterRenderFile(eio_req *req)
+void Map::EIO_AfterRenderFile(uv_work_t* req)
 {
     HandleScope scope;
 
-    return 0;
+    render_file_baton_t *closure = static_cast<render_file_baton_t *>(req->data);
+
+    TryCatch try_catch;
+
+    if (closure->error) {
+        Local<Value> argv[1] = { Exception::Error(String::New(closure->error_name.c_str())) };
+        closure->cb->Call(Context::GetCurrent()->Global(),1, argv);
+    } else {
+        Local<Value> argv[1] = { Local<Value>::New(Null()) };
+        closure->cb->Call(Context::GetCurrent()->Global(),1, argv);
+    }
+
+    if (try_catch.HasCaught()) {
+        FatalException(try_catch);
+    }
+
+    closure->m->release();
+    closure->m->Unref();
+    uv_unref(uv_default_loop());
+    closure->cb.Dispose();
+    delete closure;
+
 }
 
 Handle<Value> Map::renderLayerSync(const Arguments& args)
@@ -1323,17 +1613,17 @@ Handle<Value> Map::renderLayerSync(const Arguments& args)
 
     if (!Grid::constructor->HasInstance(obj))
       return ThrowException(Exception::TypeError(String::New("mapnik.Grid expected")));
-      
+
     std::size_t layer_idx = 0;
-    
+
     Map* m = ObjectWrap::Unwrap<Map>(args.This());
 
     std::vector<mapnik::layer> const& layers = m->map_->layers();
 
     if (args.Length() >= 2) {
-        
+
         Local<Value> layer_id = args[1];
-        
+
         if (layer_id->IsString()) {
             bool found = false;
             unsigned int idx(0);
@@ -1346,9 +1636,9 @@ Handle<Value> Map::renderLayerSync(const Arguments& args)
                     layer_idx = idx;
                     break;
                 }
-                ++idx; 
+                ++idx;
             }
-            if (!found) 
+            if (!found)
             {
                 std::ostringstream s;
                 s << "Layer name '" << layer_name << "' not found";
@@ -1357,29 +1647,29 @@ Handle<Value> Map::renderLayerSync(const Arguments& args)
         } else if (layer_id->IsNumber()) {
             layer_idx = layer_id->IntegerValue();
             std::size_t layer_num = layers.size();
-        
+
             if (layer_idx >= layer_num) {
                 std::ostringstream s;
                 s << "Zero-based layer index '" << layer_idx << "' not valid, only '"
                   << layers.size() << "' layers are in map";
                 return ThrowException(Exception::TypeError(String::New(s.str().c_str())));
-            }    
+            }
         } else {
             return ThrowException(Exception::TypeError(String::New("layer id must be a string or index number")));
         }
-    }    
-    
+    }
+
     Grid *g = ObjectWrap::Unwrap<Grid>(obj);
 
     // defaults
     double scale_factor = 1;
-    
+
     if (args.Length() >= 3) {
-    
+
         if (!args[2]->IsObject())
             return ThrowException(Exception::TypeError(
               String::New("optional second argument must be an options object")));
-    
+
         Local<Object> options = args[2]->ToObject();
 
         if (options->Has(String::New("fields"))) {
@@ -1400,7 +1690,7 @@ Handle<Value> Map::renderLayerSync(const Arguments& args)
             }
 
         }
-        
+
         if (options->Has(String::New("scale"))) {
             Local<Value> bind_opt = options->Get(String::New("scale"));
             if (!bind_opt->IsNumber())
@@ -1415,9 +1705,9 @@ Handle<Value> Map::renderLayerSync(const Arguments& args)
     {
         // copy property names
         std::set<std::string> attributes = g->get()->property_names();
-        
+
         std::string join_field = g->get()->get_key();
-        if (join_field == g->get()->id_name_) 
+        if (join_field == g->get()->key_name())
         {
             // TODO - should feature.id() be a first class attribute?
             if (attributes.find(join_field) != attributes.end())
@@ -1433,7 +1723,7 @@ Handle<Value> Map::renderLayerSync(const Arguments& args)
         mapnik::grid_renderer<mapnik::grid> ren(*m->map_,*g->get());
         mapnik::layer const& layer = layers[layer_idx];
         ren.apply(layer,attributes);
-    
+
     }
     catch (const std::exception & ex)
     {
@@ -1478,7 +1768,7 @@ Handle<Value> Map::renderSync(const Arguments& args)
             Local<Object> obj = bind_opt->ToObject();
             if (obj->IsNull() || obj->IsUndefined() || !Palette::constructor->HasInstance(obj))
               return ThrowException(Exception::TypeError(String::New("mapnik.Palette expected as second arg")));
-    
+
             palette = ObjectWrap::Unwrap<Palette>(obj)->palette();
         }
     }
@@ -1488,7 +1778,6 @@ Handle<Value> Map::renderSync(const Arguments& args)
     try
     {
         mapnik::image_32 im(m->map_->width(),m->map_->height());
-        V8::AdjustAmountOfExternalAllocatedMemory(4 * im.width() * im.height());
         mapnik::agg_renderer<mapnik::image_32> ren(*m->map_,im);
         ren.apply();
 
@@ -1557,11 +1846,11 @@ Handle<Value> Map::renderFileSync(const Arguments& args)
             if (!format_opt->IsObject())
               return ThrowException(Exception::TypeError(
                 String::New("'palette' must be an object")));
-            
+
             Local<Object> obj = format_opt->ToObject();
             if (obj->IsNull() || obj->IsUndefined() || !Palette::constructor->HasInstance(obj))
               return ThrowException(Exception::TypeError(String::New("mapnik.Palette expected as second arg")));
-    
+
             palette = ObjectWrap::Unwrap<Palette>(obj)->palette();
         }
 
@@ -1597,7 +1886,6 @@ Handle<Value> Map::renderFileSync(const Arguments& args)
         else
         {
             mapnik::image_32 im(m->map_->width(),m->map_->height());
-            V8::AdjustAmountOfExternalAllocatedMemory(4 * im.width() * im.height());
             mapnik::agg_renderer<mapnik::image_32> ren(*m->map_,im);
             ren.apply();
 
@@ -1624,6 +1912,7 @@ Handle<Value> Map::renderFileSync(const Arguments& args)
 }
 
 struct grid_t {
+    uv_work_t request;
     Map *m;
     boost::shared_ptr<mapnik::grid> grid_ptr;
     std::size_t layer_idx;
@@ -1650,7 +1939,7 @@ Handle<Value> Map::render_grid(const Arguments& args)
           << m->active()
           << " other thread(s) which is not allowed."
           << " You need to use a map pool to avoid sharing map objects between concurrent rendering";
-        std::cerr << s.str() << "\n";
+        std::clog << s.str() << "\n";
         //return ThrowException(Exception::Error(
         //  String::New(s.str().c_str())));
     }
@@ -1675,7 +1964,7 @@ Handle<Value> Map::render_grid(const Arguments& args)
     if (!args[1]->IsObject())
         return ThrowException(Exception::TypeError(
           String::New("options must be an object, eg {key: '__id__', resolution : 4, fields: ['name']}")));
-    
+
     Local<Object> options = args[1]->ToObject();
 
     std::string join_field("__id__");
@@ -1701,6 +1990,7 @@ Handle<Value> Map::render_grid(const Arguments& args)
     }
 
     grid_t *closure = new grid_t();
+    closure->request.data = closure;
 
     if (!closure) {
         V8::LowMemoryNotification();
@@ -1719,7 +2009,7 @@ Handle<Value> Map::render_grid(const Arguments& args)
     closure->error = false;
     closure->cb = Persistent<Function>::New(Handle<Function>::Cast(callback));
     closure->num_fields = 0;
-    
+
     unsigned int grid_width = m->map_->width()/step;
     unsigned int grid_height = m->map_->height()/step;
 
@@ -1748,22 +2038,22 @@ Handle<Value> Map::render_grid(const Arguments& args)
         }
     }
 
-    eio_custom(EIO_RenderGrid2, EIO_PRI_DEFAULT, EIO_AfterRenderGrid2, closure);
-    ev_ref(EV_DEFAULT_UC);
+    uv_queue_work(uv_default_loop(), &closure->request, EIO_RenderGrid2, EIO_AfterRenderGrid2);
     m->acquire();
     m->Ref();
+    uv_ref(uv_default_loop());
     return Undefined();
 
 }
 
 
-int Map::EIO_RenderGrid2(eio_req *req)
+void Map::EIO_RenderGrid2(uv_work_t* req)
 {
 
     grid_t *closure = static_cast<grid_t *>(req->data);
 
     std::vector<mapnik::layer> const& layers = closure->m->map_->layers();
-    
+
     if (!closure->layer_name.empty()) {
         bool found = false;
         unsigned int idx(0);
@@ -1776,29 +2066,29 @@ int Map::EIO_RenderGrid2(eio_req *req)
                 closure->layer_idx = idx;
                 break;
             }
-            ++idx; 
+            ++idx;
         }
-        if (!found) 
+        if (!found)
         {
             std::ostringstream s;
             s << "Layer name '" << layer_name << "' not found";
             closure->error = true;
             closure->error_name = s.str();
-            return 0;
+            return;
         }
     }
-    else 
+    else
     {
         std::size_t layer_num = layers.size();
         std::size_t layer_idx = closure->layer_idx;
-    
+
         if (layer_idx >= layer_num) {
             std::ostringstream s;
             s << "Zero-based layer index '" << layer_idx << "' not valid, only '"
               << layers.size() << "' layers are in map";
             closure->error = true;
             closure->error_name = s.str();
-            return 0;
+            return;
         }    
     }
 
@@ -1806,8 +2096,8 @@ int Map::EIO_RenderGrid2(eio_req *req)
     std::set<std::string> attributes = closure->grid_ptr->property_names();
 
     std::string const& join_field = closure->join_field;
-    
-    if (join_field == closure->grid_ptr->id_name_) 
+
+    if (join_field == closure->grid_ptr->key_name())
     {
         // TODO - should feature.id() be a first class attribute?
         if (attributes.find(join_field) != attributes.end())
@@ -1836,18 +2126,14 @@ int Map::EIO_RenderGrid2(eio_req *req)
         closure->error = true;
         closure->error_name = "Unknown error occured, please file bug";
     }
-
-    return 0;
-
 }
 
 
-int Map::EIO_AfterRenderGrid2(eio_req *req)
+void Map::EIO_AfterRenderGrid2(uv_work_t* req)
 {
     HandleScope scope;
 
     grid_t *closure = static_cast<grid_t *>(req->data);
-    ev_unref(EV_DEFAULT_UC);
 
     TryCatch try_catch;
 
@@ -1861,7 +2147,7 @@ int Map::EIO_AfterRenderGrid2(eio_req *req)
         Local<Array> grid_array = Array::New();
         std::vector<mapnik::grid::lookup_type> key_order;
         node_mapnik::grid2utf<mapnik::grid>(*closure->grid_ptr,grid_array,key_order);
-    
+
         // convert key order to proper javascript array
         Local<Array> keys_a = Array::New(key_order.size());
         std::vector<std::string>::iterator it;
@@ -1870,7 +2156,7 @@ int Map::EIO_AfterRenderGrid2(eio_req *req)
         {
             keys_a->Set(i, String::New((*it).c_str()));
         }
-    
+
         // gather feature data
         Local<Object> feature_data = Object::New();
         if (closure->num_fields > 0) {
@@ -1880,7 +2166,7 @@ int Map::EIO_AfterRenderGrid2(eio_req *req)
                            /*closure->join_field,
                            closure->grid_ptr->property_names()*/);
         }
-        
+
         // Create the return hash.
         Local<Object> json = Object::New();
         json->Set(String::NewSymbol("grid"), grid_array);
@@ -1896,8 +2182,8 @@ int Map::EIO_AfterRenderGrid2(eio_req *req)
 
     closure->m->release();
     closure->m->Unref();
+    uv_unref(uv_default_loop());
     closure->cb.Dispose();
     delete closure;
-    return 0;
 }
 
